/*

    ESTE PROGRAMA SE USA PARA CREAR UNA APLICACION PORTABLE
    A PARTIR DE UN EJECUTABLE DE GNU/LINUX.

    CREADO POR HAYLEM CANDELARIO BAUZÁ.
             SOFTWARE LIBRE GPLV3+

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

void preparework(char const * nombreap);
void createqtconf(char const *nombreap);
void createautorun(char const *nombreap);
void createuninst(char const *nombreap);
void createinst(char const *nombreap);
void copylib(char const *nombreap);
void creategz(char const *nombreap);
void createappimage(char const *nombreapp);
void cleanlibs(char const *nombreap);
void present(void);
int verifyarch(void);
void createapprun(char const *nombreprog);
void createappimagedesktop(char const *nombreapp, char const *icono, char const *categ);

int main(int argc, char *argv[])
{
  char nombreap[80] = {'\0'};

  present();
  puts("");
  printf("Introduzca el nombre del ejecutable exacto de la aplicacion-> ");
  fscanf(stdin, "%s", &nombreap);
  puts("");

  /* Preparar el directorio de trabajo */
  preparework(nombreap);

  /* Zona de dependencias */
  puts("");
  puts("\033[36m");
  puts("Todo esta preparado. Copie el ejecutable de su aplicación");
  puts("en la raíz de la carpeta que se ha creado para tí ");
  puts("con el mismo nombre del ejecutable. Si es una aplicación QT,");
  puts("recuerde copiar el contenido de la carpeta plugins dentro de plugins");
  puts("en la raiz de la carpeta de su programa.");
  puts("Usualmente esta carpeta se encuentra en /usr/lib/x86_64-linux-gnu/qt5/plugins");
  puts("para aplicaciones QT5.");
  puts("Luego escriba . y de ENTER para continuar con el proceso de empaquetado.");
  puts("");
  puts("\033[0m");


  while(getchar() != '.');

  /* Copiar librerias */
  copylib(nombreap);

  /* Crear empaquetado */
  puts("\033[31m");
  puts("Empaquetando .gz...");
  creategz(nombreap);
  puts("Empaquetando appimage...");
  createappimage(nombreap);
  puts("\033[0m");

}


void preparework(char const * nombreap)
{
  char plugdir[80] = {'\0'};
  char bindir[80] = {'\0'};
  char libdir[80] = {'\0'};


  strcpy(plugdir, nombreap);
  strcat(plugdir, "/");
  strcat(plugdir, "plugins");

  /* Limpiar librerias antiguas. */
  cleanlibs(nombreap);


  /* Crear directorio de trabajo. */
  mkdir(nombreap, 0777);

  /* Crear directorio de plugins qt. */
  mkdir(plugdir, 0777);

  strcpy(libdir, nombreap);
  strcat(libdir, "/");
  strcat(libdir, "lib");

  /* Crear directorio de librerias */
  mkdir(libdir, 0777);

  strcpy(bindir, nombreap);
  strcat(bindir, "/");
  strcat(bindir, "bin");

  /* Crear directorio de binarios */
  mkdir(bindir, 0777);

  /* Crear archivo de configuración qt */
  createqtconf(nombreap);

  /* Crear autorun */
  createautorun(nombreap);

  /* Crear desinstalador */
  createuninst(nombreap);

  /* Crear instalador */
  createinst(nombreap);
}

void createqtconf(char const *nombreap)
{
  FILE *f = NULL;
  char path[80] = {'\0'};

  strcpy(path, nombreap);
  strcat(path, "/");
  strcat(path, "qt.conf");

  f = fopen(path, "w");

  if(f == NULL)
  {
    puts("No se ha podido crear el archivo de configuracion qt.conf");
    exit(1);
  }
  else
  {
    /* Escribir la configuracion al archivo. */
    fprintf(f, "[Path]\n");
    fprintf(f, "Prefix=./\n");
    fprintf(f, "Plugins=plugins\n");
    fprintf(f, "Imports=qml\n");
    fprintf(f, "Qml2Imports=qml\n");
  }

  fclose(f);
}

void createautorun(char const *nombreap)
{
  FILE *f = NULL;
  char path[80] = {'\0'};

  strcpy(path, nombreap);
  strcat(path, "/");
  strcat(path, "autorun.sh");

  f = fopen(path, "w");

  if(f == NULL)
  {
    puts("No se ha podido crear el archivo autorun.sh");
    exit(1);
  }
  else
  {
    /* Escribir la configuracion al archivo. */
    fprintf(f, "#!/bin/bash\n");
    fprintf(f, "export PATH=./bin:$PATH\n");
    fprintf(f, "export LD_LIBRARY_PATH=./lib:$LD_LIBRARY_PATH\n");
    fprintf(f, "exec ./%s \"$@\"\n", nombreap);
  }

  fclose(f);
  chmod(path, 0777);
}

void createuninst(char const *nombreap)
{
  FILE *f = NULL;
  char path[80] = {'\0'};

  strcpy(path, nombreap);
  strcat(path, "/");
  strcat(path, "uninstall.sh");

  f = fopen(path, "w");

  if(f == NULL)
  {
    puts("No se ha podido crear el archivo uninstall.sh");
    exit(1);
  }
  else
  {
    /* Escribir la configuracion al archivo. */
    fprintf(f, "#!/bin/bash\n");
    fprintf(f, "rm -v /home/$USER/.local/share/applications/%s.desktop\n", nombreap);
  }

  fclose(f);
  chmod(path, 0777);

}

void createinst(char const *nombreap)
{
  FILE *f = NULL;
  char path[80] = {'\0'};
  char coment[250] = {'\0'};
  char icon[80] = {'\0'};
  char categ[80] = {'\0'};
  int  opcion = 0;
  char isterm[8] = {'\0'};
  int requireroot = 0;
  int i = 0;

  puts("Teclee un comentario corto para su programa a continuación terminado en punto:");
  fprintf(stdout, "-> ");

  while((coment[i] = getchar()) != '.')
  {
    if(coment[i] == '\n')
      coment[i] = ' ';

    ++i;
  }

  coment[i] = '\0';

  puts("");
  puts("Seleccione la categoría de su aplicación:");
  puts("	1)-Red");
  puts("	2)-Multimedia");
  puts("	3)-Utilitario/Sistema");
  puts("	4)-Oficina");

  fprintf(stdout, "-> ");
  fscanf(stdin, "%d", &opcion);

  switch(opcion)
  {
    case 1:
      strcpy(categ, "Network;WebBrowser;");
      break;
    case 2:
      strcpy(categ, "AudioVideo;Player;Recorder;");
      break;
    case 3:
      strcpy(categ, "GNOME;System;Filesystem;Settings;");
      break;
    case 4:
      strcpy(categ, "Office;WordProcessor;");
      break;
  }


  puts("");
  puts("¿ Se trata de una aplicación de consola ?:");
  puts("	1)-Si");
  puts("	2)-No");
  fprintf(stdout, "-> ");
  fscanf(stdin, "%d", &opcion);

  if (opcion == 1)
    strcpy(isterm, "true");
  else
    strcpy(isterm, "false");


  puts("");
  puts("¿ Requiere root la aplicacion ?:");
  puts("	1)-Si");
  puts("	2)-No");
  fprintf(stdout, "-> ");
  fscanf(stdin, "%d", &opcion);

  if (opcion == 1)
    requireroot = 1;
  else
    requireroot = 0;

  puts("");
  puts("Teclee el nombre de la imagen encontrada en la raiz de la carpeta de proyecto que usara como icono.");
  puts("Si solo desea generar un appimage ponga este nombre sin su extensión ej icono (recomendado png con 128x128).");
  puts("Si incluye la extensión en el nombre solo será útil para el portable en .tar.gz");
  puts("Copie de antemano esta imagen en la carpeta que tiene el nombre de la aplicacion.");
  puts("Su tamaño recomendado es 128x128 y puede ser png(recomendado) o svg, etc.");
  fprintf(stdout, "-> ");
  fscanf(stdin, "%s", &icon);

  strcpy(path, nombreap);
  strcat(path, "/");
  strcat(path, "install-no-root.sh");

  f = fopen(path, "w");

  if(f == NULL)
  {
    puts("No se ha podido crear el archivo install.sh");
    exit(1);
  }
  else
  {
    /* Escribir la configuracion al archivo. */
    fprintf(f, "#!/bin/bash\n");
    fprintf(f, "RUTACT=$(pwd)\n");
    fprintf(f, "echo \"[Desktop Entry]\">/home/$USER/.local/share/applications/%s.desktop\n", nombreap);
    fprintf(f, "echo \"Version=1.0\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);
    fprintf(f, "echo \"Name=%s\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap, nombreap);
    fprintf(f, "echo \"GenericName=%s\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap, nombreap);
    fprintf(f, "echo \"Comment=%s\">>/home/$USER/.local/share/applications/%s.desktop\n", coment, nombreap);
    fprintf(f, "echo \"Path=$RUTACT\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);

    if(!requireroot == 1)
     fprintf(f, "echo \"Exec=$RUTACT/autorun.sh\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);
    else
     fprintf(f, "echo \"Exec=su -c $RUTACT/autorun.sh\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);

    fprintf(f, "echo \"Icon=$RUTACT/%s\">>/home/$USER/.local/share/applications/%s.desktop\n", icon, nombreap);

    if(!requireroot == 1)
      fprintf(f, "echo \"Terminal=%s\">>/home/$USER/.local/share/applications/%s.desktop\n", isterm, nombreap);
    else
      fprintf(f, "echo \"Terminal=true\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);

    fprintf(f, "echo \"Type=Application\">>/home/$USER/.local/share/applications/%s.desktop\n", nombreap);
    fprintf(f, "echo \"Categories=%s\">>/home/$USER/.local/share/applications/%s.desktop\n", categ, nombreap);

  }


  fclose(f);
  chmod(path, 0777);
  createappimagedesktop(nombreap, icon, categ);

}


void copylib(char const *nombreap)
{

  char dstpath[80] = {'\0'};
  /* Este es el script en bash que copia las librerias. */
  const char *dependscript = "#!/bin/bash \n\
   rm -rvf /tmp/tmplib/ \n\
   mkdir /tmp/tmplib/ \n\
   cp -v /lib/*linux-gnu/libcrypt* /tmp/tmplib/ \n\
   function depend() { \n\
     for t in $(ldd \"$1\" |cut -d \">\" -f2|tr \"\t\" \" \");do \n\
       echo \"COPIANDO $t\" \n\
       cp -vn $t \"/tmp/tmplib/\" 2>/dev/null \n\
     done \n\
   } \n\
   for x in $(find ./ -type f);do \n\
     echo \"Leyendo $x\" \n\
     depend $x \n\
   done \n\
   for x in $(find /tmp/tmplib/ -type f);do \n\
     echo \"Leyendo $x\" \n\
     depend $x \n\
   done \n\
   for x in $(find /tmp/tmplib/ -type f);do \n\
     echo \"Leyendo $x\" \n\
     depend $x \n\
   done \n\
   for x in $(find /tmp/tmplib/ -type f);do \n\
     echo \"Leyendo $x\" \n\
     depend $x \n\
   done \n\
   for x in $(find /tmp/tmplib/ -type f);do \n\
     echo \"Leyendo $x\" \n\
     depend $x \n\
   done \n\
   rm /tmp/tmplib/libc.so.6 \n\
   rm /tmp/tmplib/libdl.so.2 \n\
   rm /tmp/tmplib/libpthread.so.0 \n\
   rm /tmp/tmplib/libtinfo* \n\
   rm /tmp/tmplib/libm.so.6 \n\
   rm /tmp/tmplib/libm.so \n\
   rm /tmp/tmplib/libm.so* \n\
   rm /tmp/tmplib/libresolv* \n\
   rm /tmp/tmplib/libGL* \n\
   rm /tmp/tmplib/libdrm.* \n\
   rm /tmp/tmplib/libfontconfig* \n\
  ";

  FILE *f = NULL;
  f = fopen("/tmp/copylibrerias", "w");
  if(f == NULL)
  {
    puts("No se pudo crear un modulo necesario /tmp/copylibrerias");
    exit(1);
  }
  else
  {
    fprintf(f, "%s", dependscript);
  }
  fclose(f);
  chmod("/tmp/copylibrerias", 0777);

  puts("\033[35m");
  system("/tmp/copylibrerias");
  puts("\033[0m");

  strcpy(dstpath, "./");
  strcat(dstpath, nombreap);
  strcat(dstpath, "/lib");
  rename("/tmp/tmplib", dstpath);
}


void  creategz(char const *nombreap)
{
  char cmd[80] = {'\0'};
  char finaltar[80] = {'\0'};
  char date[80] = {'\0'};
  FILE *ejec = NULL;


  ejec = popen("date|tr \" \" \"-\"", "r");
  if(ejec == NULL)
  {
    puts("No se pudo consultar la fecha a traves de date.");
    exit(1);
  }
  fscanf(ejec, "%s", &date);
  fclose(ejec);

  strcpy(cmd, "tar -vcpf ");
  strcat(cmd, nombreap);
  strcat(cmd, ".tar");
  strcat(cmd, " ./");
  strcat(cmd, nombreap);

  ejec = popen(cmd, "r");
  if(ejec == NULL)
  {
    puts("No se pudo ejecutar el empaquetado tar.");
    exit(1);
  }
  else
  {
    while(!feof(ejec)) fgetc(ejec);

    fclose(ejec);

    strcpy(cmd, "gzip --best ");
    strcat(cmd, nombreap);
    strcat(cmd, ".tar");

    ejec = popen(cmd, "r");
    if(ejec == NULL)
    {
      puts("No se pudo ejecutar el empaquetado gz.");
      exit(1);
    }
    else
    {
      while(!feof(ejec)) fgetc(ejec);

      fclose(ejec);

    }
  }

  strcpy(finaltar, "./");
  strcat(finaltar, nombreap);
  strcat(finaltar, "-");
  strcat(finaltar, date);
  strcat(finaltar, ".gz");


  strcpy(finaltar, "mv -v ./");
  strcat(finaltar, nombreap);
  strcat(finaltar, ".tar.gz");
  strcat(finaltar, " ");
  strcat(finaltar, "./");
  strcat(finaltar, nombreap);
  strcat(finaltar, "-");
  strcat(finaltar, date);
  strcat(finaltar, ".tar.gz");

  system(finaltar);

  puts("");
  puts("\033[31m");
  puts("OK");
  puts("\033[0m");

}

void cleanlibs(char const *nombreap)
{
  char direct[80] = {'\0'};

  strcpy(direct, "rm -rfv ./");
  strcat(direct, nombreap);
  strcat(direct, "/lib");

  puts("\033[34m");

  system(direct);
  system("rm -fv ./*.gz");
  system("rm -fv ./*.tar");
  puts("\033[0m");
  puts("");
}

void present(void)
{
  puts("\033[36m");
  puts("	Este programa hace tus aplicaciones ");
  puts("	Redistribuibles de forma sencilla. ");
  puts("	=================================== ");
  puts("");
  puts("\033[31m");
  puts("	CREADO POR HAYLEM CANDELARIO BAUZA");
  puts("	           LICENCIA GPLV3+");
  puts("");
  puts("--- Para generar appimage correr el programa como root ---");
  puts("");
  puts("\033[0m");


}

void createappimage(char const *nombreapp)
{
  char cmd[80] = {'\0'};
  char finaltar[80] = {'\0'};
  char date[80] = {'\0'};
  FILE *ejec = NULL;


  ejec = popen("date|tr \" \" \"-\"", "r");
  if(ejec == NULL)
  {
    puts("No se pudo consultar la fecha a traves de date.");
    exit(1);
  }
  fscanf(ejec, "%s", &date);
  fclose(ejec);

  createapprun(nombreapp);

  /* Verificar arquitectura */
  if(verifyarch() == 64)
  {
    /* Crear appimage a 64 bits */
    strcpy(cmd, " ARCH=x86_64 appimage64.appimage ");
    strcat(cmd, "./");
    strcat(cmd, nombreapp);
    system(cmd);
  }

  puts("");
  puts("\033[31m");
  puts("OK");
  puts("\033[0m");

}

int verifyarch(void)
{
  if(system("uname -a|cut -d\" \" -f9|grep x86_64") == 0)
    return 64;
  else
    return 32;
}

void createapprun(char const *nombreprog)
{
  char cmd[80] = {'\0'};
  FILE *apprun = NULL;

  strcpy(cmd, "./");
  strcat(cmd, nombreprog);
  strcat(cmd, "/AppRun");

  apprun = fopen(cmd, "w");
  if(apprun == NULL)
    puts("No se pudo crear el AppRun para el appimage.");
  else
  {
   fprintf(apprun, "#!/bin/bash\n");
   fprintf(apprun, "export APPDIR=\"$(dirname \"$(readlink -f \"$0\")\")\"\n");
   fprintf(apprun, "export PATH=\"$APPDIR/:$PATH\"\n");
   fprintf(apprun, "export LD_LIBRARY_PATH=\"$APPDIR/:$LD_LIBRARY_PATH\"\n");
   fprintf(apprun, "export XDG_DATA_DIRS=\"$APPDIR/:$XDG_DATA_DIRS\"\n");
   fprintf(apprun, "\"$APPDIR\"/%s \"$@\"\n", nombreprog);
   fclose(apprun);
   chmod(cmd, 0777);
  }
}

void createappimagedesktop(char const *nombreapp, char const *icono, char const *categ)
{
  char cmd[80] = {'\0'};
  FILE *appdsk = NULL;

  strcpy(cmd, "./");
  strcat(cmd, nombreapp);
  strcat(cmd, "/");
  strcat(cmd, nombreapp);
  strcat(cmd, ".desktop");

  appdsk = fopen(cmd, "w");
  if(appdsk == NULL)
    puts("No se pudo crear el archivo desktop  para el appimage.");
  else
  {
   fprintf(appdsk, "[Desktop Entry]\n");
   fprintf(appdsk, "Name=%s\n", nombreapp);
   fprintf(appdsk, "Exec=%s\n", nombreapp);
   fprintf(appdsk, "Icon=%s\n", icono);
   fprintf(appdsk, "Type=Application\n");
   fprintf(appdsk, "Categories=%s\n", categ);
   fclose(appdsk);
   chmod(cmd, 0777);
  }

}
